// src/shim.js
import * as React from "react";

// src/useWorker.ts
import { useEffect, useRef, useCallback, useState } from "react";
var PROMISE_RESOLVE = "resolve";
var PROMISE_REJECT = "reject";
var WORKER_STATUS = /* @__PURE__ */ ((WORKER_STATUS2) => {
  WORKER_STATUS2["PENDING"] = "PENDING";
  WORKER_STATUS2["RUNNING"] = "RUNNING";
  WORKER_STATUS2["SUCCESS"] = "SUCCESS";
  WORKER_STATUS2["ERROR"] = "ERROR";
  WORKER_STATUS2["TIME_OUT"] = "TIME_OUT";
  return WORKER_STATUS2;
})(WORKER_STATUS || {});
var DEFAULT_OPTIONS = {
  autoTerminate: false
};
function useWorker(createWorker, options = DEFAULT_OPTIONS) {
  const createWorkerRef = useRef(createWorker);
  const [workerStatus, _setWorkerStatus] = useState("PENDING" /* PENDING */);
  const workerRef = useRef();
  const isRunningRef = useRef(false);
  const promiseRef = useRef({
    [PROMISE_RESOLVE]: (value) => {
    },
    [PROMISE_REJECT]: (error) => {
    }
  });
  const timerRef = useRef(null);
  const setWorkStatus = useCallback((status) => {
    isRunningRef.current = status === "RUNNING" /* RUNNING */;
    _setWorkerStatus(status);
  }, []);
  const callWorker = useCallback((...args) => {
    return new Promise((resolve, reject) => {
      var _a;
      promiseRef.current = {
        [PROMISE_RESOLVE]: resolve,
        [PROMISE_REJECT]: reject
      };
      (_a = workerRef.current) == null ? void 0 : _a.postMessage(args);
    });
  }, []);
  useEffect(() => {
    createWorkerRef.current = createWorker;
  });
  function generateWorker() {
    if (!createWorkerRef.current) {
      return;
    }
    const worker = createWorkerRef.current();
    workerRef.current = worker;
    if (options.timeout && options.timeout > 0) {
      if (timerRef.current) {
        clearTimeout(timerRef.current);
      }
      timerRef.current = setTimeout(() => {
        promiseRef.current[PROMISE_REJECT](new Error(`the worker expect finished in ${options.timeout}ms`));
        onWorkEnd("TIME_OUT" /* TIME_OUT */);
      }, options.timeout);
    }
    worker.onmessage = ({ data }) => {
      if (workerStatus === "TIME_OUT" /* TIME_OUT */) {
        return;
      }
      if (data[0] === "SUCCESS" /* SUCCESS */) {
        promiseRef.current[PROMISE_RESOLVE](data[1]);
        onWorkEnd("SUCCESS" /* SUCCESS */);
      } else {
        promiseRef.current[PROMISE_REJECT](data[1]);
        onWorkEnd("ERROR" /* ERROR */);
      }
    };
    worker.onerror = (error) => {
      if (workerStatus === "TIME_OUT" /* TIME_OUT */) {
        return;
      }
      promiseRef.current[PROMISE_REJECT](error);
      onWorkEnd("ERROR" /* ERROR */);
    };
  }
  function onWorkEnd(workStatus) {
    setWorkStatus(workStatus);
    if (options.autoTerminate) {
      killWorker();
    }
    if (timerRef.current) {
      clearTimeout(timerRef.current);
      timerRef.current = null;
    }
  }
  function killWorker() {
    var _a;
    (_a = workerRef.current) == null ? void 0 : _a.terminate();
    workerRef.current = void 0;
    promiseRef.current = {
      [PROMISE_RESOLVE]: (value) => {
      },
      [PROMISE_REJECT]: (error) => {
      }
    };
    setWorkStatus("PENDING" /* PENDING */);
  }
  const workerRunner = useCallback((...fnArgs) => {
    if (isRunningRef.current) {
      console.warn("only one worker can be running at a time");
      return Promise.reject();
    }
    if (!createWorkerRef.current) {
      return Promise.reject();
    }
    generateWorker();
    setWorkStatus("RUNNING" /* RUNNING */);
    return callWorker(...fnArgs);
  }, []);
  const workerController = {
    workerStatus,
    killWorker
  };
  return { workerRunner, workerController };
}

// src/exportWorker.ts
function exportWorker(fn) {
  self.onmessage = async function({ data }) {
    try {
      self.postMessage(["SUCCESS" /* SUCCESS */, await fn(...data)]);
    } catch (e) {
      self.postMessage(["ERROR" /* ERROR */, e]);
    }
  };
  return fn;
}

// src/index.ts
var src_default = useWorker;
export {
  WORKER_STATUS,
  src_default as default,
  exportWorker
};
